/* Mprog.cpp : Monitor Test Patterns by Jed Margolin  Windows Version 1.0

   Copyright 2001 Jed Margolin

   Permision is given to individuals to use this code for their own use
   as long as credit is given to the author.


   Any commercial use requires permission of the author
*/

#include "stdafx.h"
#include <stdio.h>			// Header File For Standard Input/Output
#include <windows.h>		// Header File For Windows
#include <math.h>			// Header File for Math Functions
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>		// Header File For The Glaux Library
#include "mprog.h"			// my include file
#include "objects.h"		// object pictures and connection lists

extern HDC			hDC;		// Private GDI Device Context
extern HGLRC		hRC;		// Permanent Rendering Context
extern HWND		    hWnd;		// Holds Our Window Handle
extern HINSTANCE	hInstance;	// Holds The Instance Of The Application

extern LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
extern GLvoid ReSizeGLScene(GLsizei,GLsizei);
extern int InitGL(GLvoid);
extern GLvoid KillGLWindow(GLvoid);
extern BOOL CreateGLWindow(char*,int,int,int,bool);
extern glRasterPos(float,float);

extern bool	active;		// Window Active Flag
extern bool	fullscreen;	// Fullscreen Flag
extern int kbhit,key;

char quit=FALSE ;
char first_update=TRUE ;
int main_sel = 0;

void init_matrix(void);
extern GLvoid glPrint(const char *fmt, ...);   // Where The Printing Happens
void gotoxy(int,int);
//----------------------------------------
void init_uv(void);
unsigned char tchars[48];
float ax,ay,az,bx,by,bz,cx,cy,cz,xt,yt,zt,xp,yp,zp;
int *adr_ptr,obj;
float normx[100],normy[100],normz[100];
float cs,sn;			// sine and cosine of the rotation angle
//----------------------------------------
#define VK_PAGEUP 33
#define VK_PAGEDN 34

void draw_menu(void);
int do_graphics(void);
//=============================================================================
//=============================================================================
// Initialize matrix unit vectors
void init_matrix(void)
{
   init_uv();			// initialize unit vectors

   cs = 16334./16384.;		// cosine(4.4772)
   sn = 1279./16384.;		// sine(4.4772) 
}
//=============================================================================
//=============================================================================
// Redraw the Main Menu 
void draw_menu()
{
   int i;

   glLoadIdentity();						// Reset The Current Modelview Matrix
   glTranslatef(0.0f,0.0f,-12.0f);	
   glClearColor(0.0f,0.0f,0.0f,0.5f);		// black background
   glClear(GL_COLOR_BUFFER_BIT);			// Clear Screen

   //   print_header();

   for (i=0; i<NUM_ENTRIES; i++)
   {
      if(i!= main_sel)   { glColor3f(0.0f,1.0f,0.0f);}
      else			     { glColor3f(1.0f,1.0f,0.0f);}	
      gotoxy(mtable[i].xpos,mtable[i].ypos);
	  glPrint(mtable[i].msg);			// Print GL Text To The Screen
   }

   glColor3f(1.0f,1.0f,1.0f);

   gotoxy(XM,22);
   glPrint("Press UP Arrow and DOWN Arrow to navigate the menu.");
 
   gotoxy(XM,23);
   glPrint("Press Enter to make a selection.");

   gotoxy(XM,24);
   glPrint("Press F1 to toggle between full screen and window.");

}
//=============================================================================
// main exit
int mexit()
{
   return(1);
}
//=============================================================================
// Translate rows and columns to screen positions to put out text
void gotoxy(int xp, int yp)
{
   float xpos,ypos;
   xpos = -6.5f+xp/6.0f;		
   ypos = +5.5f-(yp/2.4f);		
   glRasterPos2f(xpos,ypos);	
}
//=============================================================================
// Initialize Unit Vectors
void init_uv()
{
   ax = 1.00;
   ay = 0.0;
   az = 0.0;
   bx = 0.0;
   by = 1.00;
   bz = 0.0;
   cx = 0.0;
   cy = 0.0;
   cz = 1.00;
}
// ------------------------------------------------------------------------ 
// ------------------------------------------------------------------------ 
// Object's frame of reference 
void roll(float cs, float sn)
{
   float temp;

   temp = ay*cs - az*sn;
   az   = ay*sn + az*cs;
   ay = temp;

   temp = by*cs - bz*sn;
   bz   = by*sn + bz*cs;
   by = temp;

   temp = cy*cs - cz*sn;
   cz   = cy*sn + cz*cs;
   cy = temp;
}
// ------------------------------------------------------------------------ 
// Object's frame of reference 
void pitch(float cs, float sn)
{
   float temp;

   temp = az*cs - ax*sn;
   ax   = az*sn + ax*cs;
   az = temp;

   temp = bz*cs - bx*sn;
   bx   = bz*sn + bx*cs;
   bz = temp;

   temp = cz*cs - cx*sn;
   cx   = cz*sn + cx*cs;
   cz = temp;
}
/* ------------------------------------------------------------------------ */
// Object's frame of reference/
void yaw(float cs, float sn)
{
   float temp;

   temp = ax*cs - ay*sn;
   ay   = ax*sn + ay*cs;
   ax = temp;

   temp = bx*cs - by*sn;
   by   = bx*sn + by*cs;
   bx = temp;

   temp = cx*cs - cy*sn;
   cy   = cx*sn + cy*cs;
   cx = temp;
}
// ------------------------------------------------------------------------
//------------------------------------------------------------------------ 
// Observer's frame of reference
void obs_yaw(float cs, float sn)
{
   float temp;

   temp = ax*cs - bx*sn;
   bx   = ax*sn + bx*cs;
   ax = temp;

   temp = ay*cs - by*sn;
   by   = ay*sn + by*cs;
   ay = temp;

   temp = az*cs - bz*sn;
   bz   = az*sn + bz*cs;
   az = temp;
}
// ------------------------------------------------------------------------ 
// Observer's frame of reference
void obs_roll(float cs, float sn)
{
   float temp;

   temp = bx*cs - cx*sn;
   cx   = bx*sn + cx*cs;
   bx = temp;

   temp = by*cs - cy*sn;
   cy   = by*sn + cy*cs;
   by = temp;

   temp = bz*cs - cz*sn;
   cz   = bz*sn + cz*cs;
   bz = temp;
}
// ------------------------------------------------------------------------
// Observer's frame of reference 
void obs_pitch(float cs, float sn)
{
   float temp;

   temp = cx*cs - ax*sn;
   ax   = cx*sn + ax*cs;
   cx = temp;

   temp = cy*cs - ay*sn;
   ay   = cy*sn + ay*cs;
   cy = temp;

   temp = cz*cs - az*sn;
   az   = cz*sn + az*cs;
   cz = temp;
}
// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
// Matrix Multiply - Transposed
void matrix_rmult()
{
   xp = ax*(*adr_ptr) + bx*( *(adr_ptr+1) ) + cx*( *(adr_ptr+2)) + xt;
   yp = ay*(*adr_ptr) + by*( *(adr_ptr+1) ) + cy*( *(adr_ptr+2)) + yt;
   zp = az*(*adr_ptr) + bz*( *(adr_ptr+1) ) + cz*( *(adr_ptr+2)) + zt;
}
// ------------------------------------------------------------------------ 
// ------------------------------------------------------------------------ 
// Matrix Multiply - Regular
void matrix_mult()
{
   xp = ax*(*adr_ptr) + ay*( *(adr_ptr+1) ) + az*( *(adr_ptr+2)) + xt;
   yp = bx*(*adr_ptr) + by*( *(adr_ptr+1) ) + bz*( *(adr_ptr+2)) + yt;
   zp = cx*(*adr_ptr) + cy*( *(adr_ptr+1) ) + cz*( *(adr_ptr+2)) + zt;
}
// ------------------------------------------------------------------------ 
// ------------------------------------------------------------------------ 
// Matrix Multiply - Regular but with translation
void matrix_mult2()
{
   xp = ax*(*adr_ptr) + ay*( *(adr_ptr+1) ) + az*( *(adr_ptr+2));
   yp = bx*(*adr_ptr) + by*( *(adr_ptr+1) ) + bz*( *(adr_ptr+2));
   zp = cx*(*adr_ptr) + cy*( *(adr_ptr+1) ) + cz*( *(adr_ptr+2));
}
//=============================================================================
//=============================================================================
// My program entry
int myprogram()
{
   BOOL done;
   static BOOL object_done=TRUE, exit_flag=FALSE;
	

   done = FALSE;

   if(object_done)     { draw_menu();}
   else 	         
   { 
	   object_done = do_graphics();
       return(done);
   }

   if(kbhit)					// Was key pressed?
   {
      kbhit = FALSE;			// Reset the flag 

	  switch(key)			    // look at the key pressed
	  { 
	     case VK_ESCAPE:
		 {
		    return(1);
		 }

		 case VK_UP:
		 {
			main_sel -= 1;
	        if(main_sel<0)   { main_sel = NUM_ENTRIES - 1; }
	        draw_menu();
			break;
		 }

		 case VK_DOWN:
		 {
			main_sel += 1;
	        main_sel %= NUM_ENTRIES;
	        draw_menu();
			break;
		 }

		 case VK_RETURN:
		 {

            if(main_sel==0)  { return(1); }  // main exit
			object_done = FALSE;		// start object program next time
	     	return(done);
		 }

		 case VK_F1:				// Is F1 Being Pressed?
		 {
		    KillGLWindow();						// Kill Our Current Window
		    fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
		    // Recreate Our OpenGL Window
		    if(fullscreen)
			{ 
			   if (!CreateGLWindow("Monitor Patterns by Jed Margolin",640,480,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		    else
			{
		       if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",600,400,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		 }

      }		// end switch
   }	// end kbhit		

   return(done);
}
//=======================================================================
// Do Graphics 
int do_graphics()
{
   BOOL done;
   int state;
   static int first_time = 1;
   
   if(first_time)		
   { 
	  init_matrix(); 
	  first_time = 0;
   }

   done = FALSE;

   obj = mtable[main_sel].obj;  // which object are we doing?
   xt = mtable[main_sel].xt;	// translation value for that object

   // do the graphics program as called for in the table
   mtable[main_sel].pgm();

   if(kbhit)					// Was key pressed?
   {
      kbhit = FALSE;			// Reset the flag 

	  switch(key)			    // look at the key pressed
	  { 
	     case VK_ESCAPE:
		 { 
		    done = TRUE;		// ESC Signalled A Quit
			first_time = 1;		// initialize next time
			break;
		 }

		 case VK_RETURN:		// Was RETURN Pressed?
		 {
		    done=TRUE;			// RETURN Signalled A Quit
			first_time = 1;		// initialize next time
			break;
		 }

		 case VK_LEFT:				// roll left
		 { 
		     state =  GetKeyState(VK_CONTROL);
		     if(state>=0)
			 {
			    roll(cs,-sn);		// roll object						
			 }
		     else
			 {
			    obs_roll(cs,-sn);	// roll universe
					    
			 }
						
		     break;
		 }

		 case VK_RIGHT:				// roll right
		 { 
		     state =  GetKeyState(VK_CONTROL);
		     if(state>=0)
			 {
			    roll(cs,sn);		// roll object						
			 }
		     else
			 {
			    obs_roll(cs,sn);	// roll universe
					    
			 }
						
		     break;
		 }
		 
		 case VK_UP:				// pitch up
		 { 
		     state =  GetKeyState(VK_CONTROL);
		     if(state>=0)
			 {
			    pitch(cs,-sn);		// pitch object						
			 }
		     else
			 {
			    obs_pitch(cs,-sn);	// pitch universe
					    
			 }
						
		     break;
		 }

		 case VK_DOWN:				// pitch down
		 { 
		     state =  GetKeyState(VK_CONTROL);
		     if(state>=0)
			 {
			    pitch(cs,sn);		// pitch object						
			 }
		     else
			 {
			    obs_pitch(cs,sn);	// pitch universe
					    
			 }
						
		     break;
		 }


		 case VK_END:				// Yaw Left
		 { 
		     state =  GetKeyState(VK_CONTROL);
		     if(state>=0)
			 {
			    yaw(cs,sn);		// yaw object						
			 }
		     else
			 {
			    obs_yaw(cs,sn);	// yaw universe
					    
			 }
						
		     break;
		 }

		 case VK_PAGEDN:			// Yaw Right
		 { 
		     state =  GetKeyState(VK_CONTROL);
		     if(state>=0)
			 {
			    yaw(cs,-sn);			// yaw object						
			 }
		     else
			 {
			    obs_yaw(cs,-sn);		// yaw universe
					    
			 }
						
		     break;
		 }
	    
		 case VK_F1:				// Is F1 Being Pressed?
		 {
		    KillGLWindow();						// Kill Our Current Window
		    fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
		    // Recreate Our OpenGL Window
		    if(fullscreen)
			{ 
			   if (!CreateGLWindow("Demo For 3D Unit Vectors by Jed Margolin",640,480,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		    else
			{
		       if (!CreateGLWindow("Demo For 3D Unit Vectors by Jed Margolin",600,400,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		}

	  } // end switch
   } // end kbhit

 
   return(done);
}
//=============================================================================
//=====================================================================
// Wireframe - Perspective Projection
int wf_perpgm()
{
   int m,n,color,quit,numpts,numlines,last,pnew;


   glClear(GL_COLOR_BUFFER_BIT);
   glLoadIdentity();					// Reset the current Modelview Matrix
   glTranslatef(0.0f,0.0f,-6.0f);			

// First transform and project the objects
// xt is set globally 
   yt = 0.;
   zt = 0.; 

   adr_ptr = objtable[obj].pts+1;	// address pointer for an object's points 
   numpts = objtable[obj].pts[0];   // first entry is the number of points 

   for(m=0; m<numpts; m++)
   {
      matrix_mult();
      outx[m] = yp/xp*6.0f;			// perspective projected points
      outy[m] = zp/xp*6.0f;
//	  outx[m] = yp/400.0f;			// orthographic projected points
//	  outy[m] = zp/400.0f;
      adr_ptr = adr_ptr + 3;
   }

// Draw the lines 
   
// objtable[obj].lines[n] = CLINES[n] is an index number to the entry
//   containing the drawing list point table. 

   n = 0;
   numlines = objtable[obj].lines[n++]; 

   for(m=0; m<numlines; m++)
   {
      if( objtable[obj].lines[n] == -2)      { break; }

      color = objtable[obj].lines[n++];
  	  glColor3f(ctable[color].r,
			    ctable[color].g,
				ctable[color].b);

	  last = objtable[obj].lines[n++];		// First point

      quit = FALSE;
      while(!quit)           
      {
	     pnew = objtable[obj].lines[n++];		// Drawto

		 glBegin(GL_LINES);				
	     glVertex2f(outx[last],outy[last]);
	     glVertex2f(outx[pnew],outy[pnew]);
	     glEnd();

		 last = pnew;

	     if(objtable[obj].lines[n] == -1)
		 {
	        quit = TRUE;
	        n++;
		 }           // end of if
	  }			// end of while 
   }		// end of for 

   return(0);
} 	
//========================================================================
//=====================================================================
// Wireframe - Orthographic Projection
int wf_orthpgm()
{
   int m,n,color,quit,numpts,numlines,last,pnew;

   glClear(GL_COLOR_BUFFER_BIT);
   glLoadIdentity();					// Reset the current Modelview Matrix
   glTranslatef(0.0f,0.0f,-6.0f);			

// First transform and project the objects
// xt is set globally 
//   yt = 0.;
//   zt = 0.; 

   adr_ptr = objtable[obj].pts+1;	// address pointer for an object's points 
   numpts = objtable[obj].pts[0];   // first entry is the number of points 

   for(m=0; m<numpts; m++)
   {
      matrix_mult();
//      outx[m] = yp/xp*6.0f;		// perspective projected points
//      outy[m] = zp/xp*6.0f;
	  outx[m] = yp/400.0f;			// orthographic projected points
	  outy[m] = zp/400.0f;
      adr_ptr = adr_ptr + 3;
   }

// Draw the lines 
   
// objtable[obj].lines[n] = CLINES[n] is an index number to the entry
//   containing the drawing list point table. 

   n = 0;
   numlines = objtable[obj].lines[n++]; 

   for(m=0; m<numlines; m++)
   {
      if( objtable[obj].lines[n] == -2)      { break; }

      color = objtable[obj].lines[n++];
  	  glColor3f(ctable[color].r,
			    ctable[color].g,
				ctable[color].b);

	  last = objtable[obj].lines[n++];		// First point

      quit = FALSE;
      while(!quit)           
      {
	     pnew = objtable[obj].lines[n++];		// Drawto

		 glBegin(GL_LINES);				
	     glVertex2f(outx[last],outy[last]);
	     glVertex2f(outx[pnew],outy[pnew]);
	     glEnd();

		 last = pnew;

	     if(objtable[obj].lines[n] == -1)
		 {
	        quit = TRUE;
	        n++;
		 }           // end of if
	  }			// end of while 
   }		// end of for 

   return(0);
} 	
//========================================================================
// Polygon - Orthographic Projection
int poly_orthpgm()
{
   int m,n,color,numpts,numfaces,quit;
   float acc;

   glClear(GL_COLOR_BUFFER_BIT);
   glLoadIdentity();					// Reset the current Modelview Matrix
   glTranslatef(0.0f,0.0f,-6.0f);			

// First transform and project the objects
// xt is set globally 
   yt = 0.;
   zt = 0.; 

   adr_ptr = objtable[obj].pts+1;	// address pointer for an object's points 
   numpts = objtable[obj].pts[0];   // first entry is the number of points 

   for(m=0; m<numpts; m++)
   {
      matrix_mult();
//    outx[m] = yp/xp*6.0f;			// perspective projected points
//    outy[m] = zp/xp*6.0f;
	  outx[m] = yp/150.0f;			// orthographic projected points
	  outy[m] = zp/150.0f;
      adr_ptr = adr_ptr + 3;
   }

// Now transform the normals 
// Each face has two normals (one in each direction)
   adr_ptr = objtable[obj].norms+1;     // address pointer to the list of an object's face normals
   numfaces = objtable[obj].norms[0];   // first entry is the number of faces

   for(m=0; m<numfaces; m++)
   {
      matrix_mult2();			// without translation
      normx[m] = xp;
      normy[m] = yp;
      normz[m] = zp;
      adr_ptr = adr_ptr + 3;
   }

/* Now do the faces and draw the polygons
   Is the face visible? 
      Do the Dot Product: (Viewer to Center Point) x Normal
      XT*(Nx-xt) + YT*(Ny-yt) + ZT*(Nz-zt)
      objtable[obj].faces[n] = CFACES[n]   
*/

   n = 1;   							// index to CFACES array 
   numfaces = objtable[obj].faces[0];   // example: CFACES[0] 
   for(m=0; m<numfaces; m++)
   {
      acc = normx[m];										// orthographic

      if(acc <0.0)	// Draw it if it is negative
      {
	     // Draw the polygon
		 color = objtable[obj].faces[n++];		// color of the face
  		 glColor3f(ctable[color].r,
		 	       ctable[color].g,
			       ctable[color].b);

		 quit = 1;
	     while(quit)           
		 {
	        glBegin(GL_POLYGON);	// GL_POLYGON
			glVertex2f(outx[objtable[obj].faces[n]],
					   outy[objtable[obj].faces[n]]);
			n++;
			 if(objtable[obj].faces[n] == -1)  
			 {
	            quit = 0;		// last vertex in the face
	            n++;
				glEnd();		// done drawing the face
			 }      
         }		// end of drawing this face 
      }     // end of this face, drawn or not
      else  { while(objtable[obj].faces[n++] != -1); }  // face not drawn

   }  	// end of number of faces 

   return(0);
}	
//=============================================================================
//=============================================================================
// Polygon - Perspective Projection
int poly_perpgm()
{
   int m,n,color,numpts,numfaces,quit,orgpt;
   float acc;


   glClear(GL_COLOR_BUFFER_BIT);
   glLoadIdentity();					// Reset the current Modelview Matrix
   glTranslatef(0.0f,0.0f,-6.0f);			

// First transform and project the objects
// xt is set globally 
   yt = 0.;
   zt = 0.; 

   adr_ptr = objtable[obj].pts+1;	// address pointer for an object's points 
   numpts = objtable[obj].pts[0];   // first entry is the number of points 

   for(m=0; m<numpts; m++)
   {
      matrix_mult();
	  axout[m] = xp;
	  ayout[m] = yp;
	  azout[m] = zp;

      outx[m] = yp/xp*6.0f;				// projected points
      outy[m] = zp/xp*6.0f;
//	  outx[m] = yp/200.0f;
//	  outy[m] = zp/200.0f;
      adr_ptr = adr_ptr + 3;
   }

// Now transform the normals without translation 
// Each face has two normals (one in each direction)
   adr_ptr = objtable[obj].norms+1;     // address pointer to the list of an object's face normals
   numfaces = objtable[obj].norms[0];   // first entry is the number of faces

   for(m=0; m<numfaces; m++)
   {
      matrix_mult2();
      normx[m] = xp;
      normy[m] = yp;
      normz[m] = zp;
      adr_ptr = adr_ptr + 3;
   }

/* Now do the faces and draw the polygons
   Is the face visible? 
     -> Do the Dot Product between the Normal and a point on its face
 
     normx[m]*axout[orgpt] + normy[m]*ayout[orgpt] + normz[m]*axout[orgpt]
  
     From the list of origin points for an object's face normals:
	    orgpt = objtable[obj].norgs[m]
*/

   n = 1;   							// index to CFACES array 
   numfaces = objtable[obj].faces[0];   // example: CFACES[0] 
   for(m=0; m<numfaces; m++)
   {
	  orgpt = objtable[obj].norgs[m];
      acc = normx[m]*axout[orgpt] + normy[m]*ayout[orgpt] + normz[m]*azout[orgpt];

      if(acc <0.0)	// Draw it if it is negative
      {
	     // Draw the polygon
		 color = objtable[obj].faces[n++];		// color of the face
  		 glColor3f(ctable[color].r,
		 	       ctable[color].g,
			       ctable[color].b);

		 quit = 0;
	     while(!quit)           
		 {
	        glBegin(GL_POLYGON);	
			glVertex2f(outx[objtable[obj].faces[n]],
					   outy[objtable[obj].faces[n]]);
			n++;

			if(objtable[obj].faces[n] == -1)  
			{
	           quit = 1;		// last vertex in the face
	           n++;
			   glEnd();		// done drawing the face
			}      
         }		// end of while (drawing this face) 
      }     // end of this face
      else  // face not drawn
	  { 
	     quit=0;
		 while(!quit)
		 { 
		    if(objtable[obj].faces[n] == -1)	  { quit = 1;}
		    n++;
		 }  
	  }

// the following code would have been more efficient, but while it works 
// in the debug .exe, it does not work in the release .exe .
//      else  { while(objtable[obj].faces[n++] != -1); }  // face not drawn
// It looks like it doesn't like to increment inside an indexed structure

   }  	// end of number of faces 

   return(0);
}	
//=============================================================================
/*
Object Data Lists
-----------------

 List of points for an object 	
	points[n] =  objtable[obj].pts[n]
 First location is number of points	


 List of wireframe point connections for an object
	lines[n] = objtable[obj].lines[n]
 First location is number of polygons
   Each polygon: Color, pt0, pt1, pt2, ..., ptn, -1


 List of faces for an object 		
	faces[n] = objtable[obj].faces[n]
 First location is number of faces
 Each face: Color, pt0, pt1, pt2, ..., ptn, -1	
 

 List of face normals for an object	
	normals[n] = objtable[obj].norms[n]
 First location is number of normals	
 Each 'normal' corresponds to a face


 List of origin points for an object's face normals
	norgs[n] = objtable[obj].norgs[n]
 Contains the same number as the number of normals

*/
//=============================================================================
// Instructions
int instr_pgm(void)
{
   int i,color;
   xt = 250 ;
   yt = -250;
   zt = -250;

   obj = 3;
   wf_orthpgm();			// wireframe program: orthographic projection

// directions
   glLoadIdentity();						// Reset The Current Modelview Matrix
   glTranslatef(0.0f,0.0f,-12.0f);	

   for(i=0; i<NUM_MSGS; i++)
   {
     color = msgtable[i].color;
	 glColor3f(lctable[color].r, lctable[color].g,lctable[color].b);

     gotoxy(msgtable[i].xpos,msgtable[i].ypos);
     glPrint(msgtable[i].msg);			// Print GL Text To The Screen
   }

   return(0);
}
//======================================================================
