// uvmath.c 
// Jed Margolin 1995, 2001
// Demonstrate 3D Unit Vector Math

#include "stdafx.h"
#include <stdio.h>			// Header File For Standard Input/Output
#include <windows.h>		// Header File For Windows
#include <math.h>			// Header File for Math Functions
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>		// Header File For The Glaux Library
#include "objects.h"

/*
extern HDC			hDC;		// Private GDI Device Context
extern HGLRC		hRC;		// Permanent Rendering Context
extern HWND		    hWnd;		// Holds Our Window Handle
extern HINSTANCE	hInstance;	// Holds The Instance Of The Application

extern LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
extern GLvoid ReSizeGLScene(GLsizei,GLsizei);
extern int InitGL(GLvoid);
extern GLvoid KillGLWindow(GLvoid);
extern BOOL CreateGLWindow(char*,int,int,int,bool);
extern glRasterPos(float,float);

extern bool	active;		// Window Active Flag
extern bool	fullscreen;	// Fullscreen Flag
extern int kbhit,key;
extern GLvoid glPrint(const char *fmt, ...);   // Where The Printing Happens
void gotoxy(int,int);


extern int main_sel;
*/

unsigned char tchars[48];
float ax,ay,az,bx,by,bz,cx,cy,cz,xt,yt,zt,xp,yp,zp;
int *adr_ptr;
int last,new;
float normx[100],normy[100],normz[100],foutx[100],fouty[100];
int polypts[200];
#include "objects.h"
// ------------------------------------------------------------------------ 
void init_uv()
{
   ax = 1.00;
   ay = 0.0;
   az = 0.0;
   bx = 0.0;
   by = 1.00;
   bz = 0.0;
   cx = 0.0;
   cy = 0.0;
   cz = 1.00;
}
// ------------------------------------------------------------------------ 
// ------------------------------------------------------------------------ 
// Object's frame of reference 
void roll(float cs, float sn)
{
   float temp;

   temp = ay*cs - az*sn;
   az   = ay*sn + az*cs;
   ay = temp;

   temp = by*cs - bz*sn;
   bz   = by*sn + bz*cs;
   by = temp;

   temp = cy*cs - cz*sn;
   cz   = cy*sn + cz*cs;
   cy = temp;
}
// ------------------------------------------------------------------------ 
// Object's frame of reference 
void pitch(float cs, float sn)
{
   float temp;

   temp = az*cs - ax*sn;
   ax   = az*sn + ax*cs;
   az = temp;

   temp = bz*cs - bx*sn;
   bx   = bz*sn + bx*cs;
   bz = temp;

   temp = cz*cs - cx*sn;
   cx   = cz*sn + cx*cs;
   cz = temp;
}
/* ------------------------------------------------------------------------ */
// Object's frame of reference/
void yaw(float cs, float sn)
{
   float temp;

   temp = ax*cs - ay*sn;
   ay   = ax*sn + ay*cs;
   ax = temp;

   temp = bx*cs - by*sn;
   by   = bx*sn + by*cs;
   bx = temp;

   temp = cx*cs - cy*sn;
   cy   = cx*sn + cy*cs;
   cx = temp;
}
// ------------------------------------------------------------------------
//------------------------------------------------------------------------ 
// Observer's frame of reference
void obs_yaw(float cs, float sn)
{
   float temp;

   temp = ax*cs - bx*sn;
   bx   = ax*sn + bx*cs;
   ax = temp;

   temp = ay*cs - by*sn;
   by   = ay*sn + by*cs;
   ay = temp;

   temp = az*cs - bz*sn;
   bz   = az*sn + bz*cs;
   az = temp;
}
// ------------------------------------------------------------------------ 
// Observer's frame of reference
void obs_roll(float cs, float sn)
{
   float temp;

   temp = bx*cs - cx*sn;
   cx   = bx*sn + cx*cs;
   bx = temp;

   temp = by*cs - cy*sn;
   cy   = by*sn + cy*cs;
   by = temp;

   temp = bz*cs - cz*sn;
   cz   = bz*sn + cz*cs;
   bz = temp;
}
// ------------------------------------------------------------------------
// Observer's frame of reference 
void obs_pitch(float cs, float sn)
{
   float temp;

   temp = cx*cs - ax*sn;
   ax   = cx*sn + ax*cs;
   cx = temp;

   temp = cy*cs - ay*sn;
   ay   = cy*sn + ay*cs;
   cy = temp;

   temp = cz*cs - az*sn;
   az   = cz*sn + az*cs;
   cz = temp;
}
// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
void matrix_rmult()
{
   xp = ax*(*adr_ptr) + bx*( *(adr_ptr+1) ) + cx*( *(adr_ptr+2)) + xt;
   yp = ay*(*adr_ptr) + by*( *(adr_ptr+1) ) + cy*( *(adr_ptr+2)) + yt;
   zp = az*(*adr_ptr) + bz*( *(adr_ptr+1) ) + cz*( *(adr_ptr+2)) + zt;
}
// ------------------------------------------------------------------------ 
// ------------------------------------------------------------------------ 
void matrix_mult()
{
   xp = ax*(*adr_ptr) + ay*( *(adr_ptr+1) ) + az*( *(adr_ptr+2)) + xt;
   yp = bx*(*adr_ptr) + by*( *(adr_ptr+1) ) + bz*( *(adr_ptr+2)) + yt;
   zp = cx*(*adr_ptr) + cy*( *(adr_ptr+1) ) + cz*( *(adr_ptr+2)) + zt;
}

